/*
      SoundMan v0.1 - Realtime frequency analyzer
      Copyright (C) 1999 Gabor Kerenyi (wom@fireball.kalmar.hu)
*/

#include <qwidget.h>
#include "sdata.h"
#include "main.h"
#include <qscrollbar.h>
#include <qpainter.h>
#include <fcntl.h>
#include <math.h>

extern cnf mconf;
QPainter p1;

SData::SData(QWidget *parent, const char *name): QWidget(parent,name)
{
setBackgroundColor(QColor(0,0,0));
setBackgroundMode(FixedColor);
setPalette(QPalette(QColor(0,0,0)));
}

void SData::mkc(int maxx,int maxy)
{
int i,j,x;
char s[10];
float xmag;

p1.setPen(QColor(mconf.col.sgr_x.r,mconf.col.sgr_x.g,mconf.col.sgr_x.b));
xmag=(double)maxx/(double)(mconf.pg.window_length);
p1.drawLine(0,maxy-30,maxx,maxy-30);
j=0;
for (i=mconf.pg.window_start;i<mconf.pg.window_start+mconf.pg.window_length;i++)
{
  if ((i%(mconf.pg.window_length/20)==0)){
	x=(int)((i-mconf.pg.window_start)*xmag);
	j++;
	if (j==3){
	   sprintf(s,"%.2fs",((float)i*(float)mconf.pg.pn)/(float)mconf.dev.freq);
	   p1.drawLine(x+40,maxy-20,x+40,maxy-30);
           p1.setPen(QColor(mconf.col.sgr_xtext.r,mconf.col.sgr_xtext.g,mconf.col.sgr_xtext.b));
	   p1.drawText(x+40-strlen(s)*3,maxy-5,s);
           p1.setPen(QColor(mconf.col.sgr_x.r,mconf.col.sgr_x.g,mconf.col.sgr_x.b));
	   j=0;
	}
	else
	   p1.drawLine(x+40,maxy-25,x+40,maxy-30);
  }
}
}

void SData::mkc_v(int maxy)
{
int i,y;
int step1,step2;
char s[10];
float ymag;

p1.setPen(QColor(255,255,255));
ymag=(double)maxy/(double)(mconf.dev.freq/2);
step1=0;
step2=0;
switch(mconf.dev.freq)
{
  case 44100: step1=500;step2=2500;break;
  case 22050: step1=200;step2=1000;break;
  case 11025: step1=100;step2=1000;break;
}
p1.setPen(QColor(mconf.col.sgr_y.r,mconf.col.sgr_y.g,mconf.col.sgr_y.b));
p1.drawLine(40,maxy,40,0);
//p1.setPen(QColor(mconf.col.sgr_y.r,mconf.col.sgr_y.g,mconf.col.sgr_y.b));
for (i=1;i<=mconf.dev.freq/2;i++)
{
  if (i%step1==0){
    y=(int)((mconf.dev.freq/2-i)*ymag);
    if (i%step2==0){
       p1.drawLine(30,y,40,y);
       sprintf(s,"%.1f",(double)i/1000.0);
       p1.setPen(QColor(mconf.col.sgr_ytext.r,mconf.col.sgr_ytext.g,mconf.col.sgr_ytext.b));
       p1.drawText(2,y+5,s);
       p1.setPen(QColor(mconf.col.sgr_y.r,mconf.col.sgr_y.g,mconf.col.sgr_y.b));
    }
    else
       p1.drawLine(35,y,40,y);
  }
}
}

void SData::draw_col(int maxy,int k,float zmag,int x)
{
int i,j,point,y;
float ymag=(float)maxy/(float)(mconf.pg.pn/2);

p1.drawLine(45,0,45,maxy);
for (i=1;i<mconf.pg.pn/2-2;i++)
{
   y=(int)((float)i*ymag);
   if (mconf.pg.logcol)
        point=(int)(log10(*((*(mconf.pg.val+k))+(mconf.pg.pn/2-i+2))+1)*zmag);
   else
        point=(int)(*((*(mconf.pg.val+k))+(mconf.pg.pn/2-i+2))*zmag);
   if (point>255){
	point=0;
   }
   if (mconf.pg.black)
      if (point<=mconf.pg.black_value)
          point=0;
   if (mconf.pg.cut)
      if (point>=mconf.pg.cut_value)
          point=255;
   if ((mconf.pg.con)&&(point!=0))
      point+=mconf.pg.con_value;
   if (point>255)
      point=255;
   p1.setPen(QColor(point,point,point));
   for (j=0;j<(float)(i+1)*ymag-y;j++)
      p1.drawPoint(x+45,y+j); 
 }
} 

void SData::paintEvent(QPaintEvent *)
{
int maxx;
int maxy,basey;
long i,j;
float zmag,xmag;

maxx=width()-30;
maxy=height()-30;
basey=maxy-15;
if (mconf.pg.val!=NULL){
   if (mconf.pg.logcol)
       zmag=(float)254/log10(mconf.spect.maxpower+1.01);
   else
       zmag=(float)254/(float)mconf.spect.maxpower;
   xmag=(float)maxx/(float)(mconf.pg.window_number/mconf.pg.zoom);
   p1.begin(this);
   p1.setBrush(QColor(100,0,0));
   p1.setBackgroundColor(QColor(100,0,0));
   mkc_v(maxy);
   mkc(maxx+30,maxy+30);
   for (j=mconf.pg.window_start;j<mconf.pg.window_start+mconf.pg.window_length;j++)
   {
      for (i=0;i<xmag;i++)
	 draw_col(maxy,j,zmag,(float)(j-mconf.pg.window_start)*xmag+i);
   }
   mkc_v(maxy);
   mkc(maxx+30,maxy+30);
   p1.end();
}
}

SData::~SData()
{
}

